.de XS
.nf
.ne 15
.sp
..
.de XE
.sp
.fi
..
.ND
.ce
kawk
.SH
DESCRIPTION
.LP
This program is the Bibliofile analogue to UNIX awk.
It provides a general purpose programming language based
on C and over 100 special functions designed for the
manipulation of Bibliofile records.
.SH
CAVEAT READER
.LP
Because many of the constructs used in kawk programming
involve back slash-character combinations, nroff processing
may introduce errors into the text.  As a last resort, check
the original of this documentation.  It may be read in raw
form with the command
.LP
.li
more $\&PUBKIN/MAN/kawk.t
..
.SH
USAGE MODES
.LP
kawk may be used either interactively or in batch mode.  The
latter is default.  The former is initiated by
.LP
kawk -i
.LP
Interactive mode commands will not be discussed here as they
closely parallel all Bibliofile programs.  Once you have
started kawk interactively, the interactive command set is
available by typing a single question mark.
.SH
BATCH MODE OPTIONS
.LP
-k file
.LP
Specify a specific key file rather than the default ".keys".
.LP
-K
.LP
Do not look for a key file.  This option allows kawk to
be used as a general purpose programming language.
.LP
-e script
.LP
Take the program from the command line variable "script."
Generally "script" will be a variable assigned in a shell
script as in this example
.XS
.so script.kwk
.XE
.IP NOTE
It can be an exercise in extreme frustration to pass programs
containing many quotation marks through the Bourne shell.
.SH
THE KAWK LANGUAGE
.LP
kawk programs are composed of one or more blocks, typically
.XS
/* declaration of variables */
{
/*  statements to execute before any reading */
    while(rdcard()) {
    /* read loop */
    }
/*  statements to execute after all files have been read */
}
.XE
.LP
Blocks are executed in sequence unless exit() is called.  The
setfile() and setofn() commands allow kawk blocks to process
different input files, apply two or more sets of commands to the
same input files, to produce work files that are processed by a
subsequent block, etc.  One possibility is shown below.  The
effect is that of a "pipe" internal to kawk.
.XS
{
    setofn("barf"); /* set output file name to barf */
    while(rdcard()) {
        /* do some processing */
        }
}
.XE
.XS
{
    setofn();           /* resets output to stdout */
    setfile("barf");    /* opens "barf" as input */
    while(rdcard()) {
        /* do some more processing */
        }
}
.XE
.IP NOTE
All variables in kawk are global and initialized once when
declared to zero.  Variables at the beginning of second and
subsequent blocks will have the value they had at the end of the
previous block.
.LP
.SH
ARITHMETIC
.LP
kawk provides all standard arithmetic operations plus several of
the extended operations available in the C programming language.
.IP +
addition
.IP -
subtraction
.IP *
multiplication
.IP /
division
.IP %
remainder (modulu)
.LP
kawk follows the rules of C precedence.  The expression
.LP
z = 5 + 3 / 2;
.LP
is evaluated as
.LP
z = 5 + (3 / 2);
.IP NOTE
Integer division does NOT yield a remainder.  Therefore, z is
assigned the value 6.
.LP
Precedence may be altered with literal parens as in
.LP
z = (5 + 3) / 2;
.SH
OPERATORS
.LP
kawk arithmetic operators are a subset of those available in C and
awk. ++ and -- are not implemented.  The combined arithmetic and
assignment operators are fully implemented.
.LP
x = 1 + x;
.LP
may be written as
.LP
x += 1;
.SH
VARIABLE DECLARATIONS AND TYPES
.LP
kawk variables must be declared explicitly.  Variables may be
char, int, or double and may be either single or arrays.
.LP
.li
int     one, two, three;    /* three simple variables */

int     four[BUFSIZ];       /* an integer array */

char    gorp, slop;         /* two simple variables */

char    burp[BUFSIZ];       /* one character string */

char    gawd[6969];         /* one ghastly array */

double  big;                /* one double precision variable */

double  slurp[16];          /* a double array */
..
.LP
When double and int types appear in the same expression, kawk
will promote the lesser precision variables to double on a pair
by pair basis.  This may lead to unexpected results unless you
pay careful attention to the rules of precedence described
above.
.LP
Currently only single dimension arrays are available.  Array
size must be given as a number except that BUFSIZ (512)
is recognized.
.LP
There is no provision for initializing arrays element by element.
Strings may be initialized by
.LP
strcpy(burp,"setting string burp");
.LP
before the array is used.
.LP
Simple variables are initialized by assignment as in
.LP
big = 9999.9999;
.LP
one = 69;
.LP
two = '\e007';
.SH
VARIABLE SIZE ARRAYS
.LP
The strsave(is) function saves strings in dynamically allocated
core.  The memory allocated can be reallocated by strfree() and
a second call to strsave().  When used with an integer/pointer
array, strsave() and strfree() give kawk a mechanism for dynamic,
multidimension arrays.
.XS
.so strsave.kwk
.XE
.IP NOTE
The strsave/strfree mechanism is somewhat fragile.  If you
try to strfree a block that was not obtained by strsave,
you will either produce a core dump or a loop.  This is NOT
a kawk bug, but a problem inherent in malloc and free, the
UNIX system functions called by kawk.
.SH
FUNCTIONS VERSUS VARIABLES
.LP
Generally the output from a function, provided it is of the
proper type, may be used anywhere a simple variable is called
for.  The following are equivalent:
.XS
    fs = filesize("barf");
    printf("barf = %d characters\n",fs);
.XS
.XS
    printf("%s = %d\n",is,filesize(is));
.XE
.SH
FORMAT CONVENTIONS
.LP
kawk formats are the same as those of C.  Floating point formats
will be accepted but produce garbage output.  There are a number
of extensions to C format conventions that ease producing output
for kroff/nroff/troff.  See the language summary below for a
list.
.SH
ESCAPE CONVENTIONS
.LP
kawk accepts standard C escapes such as \en to produce a line feed.
Sequences of the form \e000 produce the corresponding byte
equivalent.  These forms apply to both string and character
constants.  Problems will occur because a given string constant
may be subject to TWO OR MORE levels of escape expansion.
.LP
For example, to print the [knt]roff bolding command
.LP
puts("\efB");
.LP
will not work as expected because \ef is siginifcant to C and is
turned into control L (a form feed).  To get the correct results,
use
.LP
puts("\e\efB");
.LP
An even more complex case occurs with pattern replacement because
kawk processes strings according to C conventions, but certain
routines add another layer of processing, AND the amount of
processing is NOT THE SAME for the target and replacement strings
of such expressions!  For example, on the target side \eU has no
particulary meaning and will pass through kawk processing as \eU.
On the replacement side, \eU mean convert the following string to
upper case.
.LP
In this context, the only real guidance that can be given is
.IP a
curse the sob(s) who designed UNIX with only 128 characters in
mind;
.IP b
try to think of nifty dodges that do not require escape
characters;
.IP c
break tasks into simple elements, experimenting on a line by line
basis.
.sp
To experiment with critical strings, use the function
.sp
funxesc("string");
.sp
It prints control characters as @X and non-ascii characters as
\e000.  Using it with the example above would yield
.sp
@LfB
.sp
showing the conversion of \ef into control L (@L).
.IP NOTE
Before you jump to the conclusion that Bibliofile is a
crock of amateurish drek, check the consultation columns of
various PC magazines to see the horrendous scripts that are
necessary to produce similar results with "1-2-3", "dBase",
etc., IF YOU CAN PERFORM THE 
.B
DESIRED
.R
OPERATION AT
ALL. After you have done so, mail me a check for $\&495.00,
or whatever you would pay for one of these programs plus
the add-ons necessary to make it half-way usable!
.SH
THE KISS PRINCIPLE
.LP
When using kawk (or any other program) to write scripts that will
be used by nroff/troff, adhere to the KISS (keep it simple,
stupid!) principle.  Do not try to cram too much into a single
kawk program or format statement.  Try to avoid strings that have
many escapes.  If these are constants, use something like
.LP
if(condition)   printf("NZ1");
.LP
and later expand NZ1 to the appropriate nroff/troff script using
sed (or another run of kawk) that concentrates on expansions.
.SH
DIAGNOSTICS
.LP
kawk produces diagnostics both during compilation and execution.
Compilation errors usually produce a pointer of the form
.XS
length = strlent(is) + 5;
---------|
<unknown function>
.XE
Compilation stops at the first error.
.IP NOTE
If you see @I in an error message, you are seeing tabs that have
been converted to a visible presentation.
.LP
During execution each routine is checked for the proper number of
parameters in its call.  A mismatch halts execution with a listing
of the offending routine.  Other conditions that will halt execution
include
.IP
trying to open a file that does not exist;
.IP
trying to divide by zero;
.IP
trying to read or write a pipe that is not working.
.SH
KAWK FUNCTIONS
.LP
kawk has a large number of builtin functions designed for ease in
handling Bibliofile records.  In the explanation that follows,
the formal parameters for the routines are generally coded as
.IP os
an output string.  THIS SHOULD NEVER BE A $\&key VARIABLE.
.IP ns
the size of the output string in characters.
.IP is
the input string which may be a char string[size] variable, a
$\&key variable, or the output from a function that returns a
string or pointer.
.IP num
an integer variable or the output from a function that returns
the same.
.IP NOTE
Not all versions of kawk provide the same set of functions.
Because of program size limitation particularly under MSDOS, some
versions may not include the kuso and termio group of functions.
The kuso functions can be replicated using other kawk built in
functions.
.SH
SUMMARY OF THE KAWK LANGUAGE
.LP
.XS
.so kadoc4.d
.XE
.SH
LIST OF CURRENT KAWK FUNCTIONS
.XS
.sk kadoc2.d
.XE
.SH
SAMPLE KAWK PROGRAM(S)
.LP
The program given below uses [kn]roff to produce a conventional
bibliography.
.XS
.so kadoc3.d
.XE
.LP
The program given below is used to massage the various files that
are used to create this documentation.  It is a good example of
breaking a messy task into simple pieces and making multiple
passes through scratch files.
.XS
.so docfix.kwk
.XE
